# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""
spyder.utils.external
=====================

* Libraries or modules taken from external projects verbatim.
* Spyder developers can modify these modules to make them work
  better for the project. So there's no guarantee that we will
  maintain compatibility with the upstream project the code
  was taken from.
"""
